from aliyunsdkslb.request.v20140515 import DescribeLoadBalancersRequest, \
    DescribeListenerAccessControlAttributeRequest, \
    DescribeLoadBalancerAttributeRequest, DescribeHealthStatusRequest, \
    DescribeRulesRequest, DescribeRuleAttributeRequest, \
    DescribeVServerGroupsRequest, DescribeVServerGroupAttributeRequest, \
    CreateLoadBalancerRequest, DescribeRegionsRequest, \
    SetLoadBalancerTCPListenerAttributeRequest
from .base import BaseAliyunApi


class AliyunSlb(BaseAliyunApi):
    """
    阿里云SLB负载均衡相关API
        获取实例列表；
        获取指定实例配置；
        获取指定实例后端不健康主机实例ID列表；
        获取指定端口转发规则列表或指定规则；
    """

    def get_regions(self, id_only=False):
        """
        获取支持的region列表
        https://help.aliyun.com/document_detail/27584.html?spm=a2c4g.11174283.6.633.58ef11925XsImC
        :param id_only: 是否只返回region id列表
        :return:
        """
        request = DescribeRegionsRequest.DescribeRegionsRequest()
        result = self.client.do_action(request)
        result_list = result.get('Regions', {}).get('Region', [])
        if id_only:
            result_list = list(set([x.get('RegionId') for x in result_list]))
        return result_list

    def list_slb(self, name_only=False, **kwargs):
        """
        获取实例列表信息
        https://help.aliyun.com/document_detail/27582.html?spm=a2c4g.11186623.6.631.424952e51QEPuJ
        :param name_only:
        :param kwargs:
        :return:
        """
        result_list = []
        request = DescribeLoadBalancersRequest.DescribeLoadBalancersRequest()
        regions = self.get_regions(id_only=True)
        for region in regions:
            request.set_PageSize(100)
            request.set_PageNumber(1)
            for k, v in kwargs.items():
                request.add_query_param(k, v)
            request.add_query_param('RegionId', region)
            result = self.client.do_action(request)
            result_list.extend(result.get('LoadBalancers', {}).get('LoadBalancer', []))

            total_count = float(result.get('TotalCount', 0))
            if not total_count:
                continue
            page_count = float(result.get('PageSize', 1))
            _p = total_count / page_count
            if _p > 1:
                for _p_num in range(2, int(_p) + 2):
                    request.set_PageNumber(_p_num)
                    result = self.client.do_action(request)
                    result_list.extend(result.get('LoadBalancers').get('LoadBalancer'))

        if name_only:
            result_list = [i['InstanceName'] for i in result_list]
        return result_list

    def get_slb_list(self):
        """
        :return: SLB实例列表
        """
        request = DescribeLoadBalancersRequest.DescribeLoadBalancersRequest()
        request.set_accept_format('json')
        result = self.client.do_action(request)['LoadBalancers']['LoadBalancer']
        return result

    def get_slb_config(self, slb_id):
        """
        :params:
            SLB实例ID
        :return:
            SLB ID对应实例配置字典
        """
        request = DescribeLoadBalancerAttributeRequest.DescribeLoadBalancerAttributeRequest()
        request.set_accept_format('json')
        request.set_LoadBalancerId(slb_id)
        result = self.client.do_action(request)
        return result

    def get_listener_whitelist(self, slb_id, listener_port):
        """
        :params:
            SLB实例ID 前端监听端口号
        :return:
            白名单列表字符串
        """
        request = DescribeListenerAccessControlAttributeRequest \
            .DescribeListenerAccessControlAttributeRequest()
        request.set_accept_format('json')
        request.set_LoadBalancerId(slb_id)
        request.set_ListenerPort(listener_port)
        result = self.client.do_action(request)
        # pylint:disable=C0303
        return result['SourceItems'] \
            if result['AccessControlStatus'] == \
               'open_white_list' else None

    def get_unhealthy_backends(self, slb_id, listener_port):
        """
        :params:
            SLB实例ID 前端监听端口号
        :return:
            后端不健康实例ID列表
        """
        request = DescribeHealthStatusRequest.DescribeHealthStatusRequest()
        request.set_accept_format('json')
        request.set_LoadBalancerId(slb_id)
        request.set_ListenerPort(listener_port)
        result = self.client \
            .do_action(request)['BackendServers']['BackendServer']
        return [backend['ServerId'] for backend in result \
                if backend['ServerHealthStatus'] == 'abnormal']

    def get_redirect_rules(self, slb_id, listener_port, rule_id='all'):
        """
        :params:
            SLB实例ID 后端监听端口号 转发规则ID
        :return:
            转发规则列表
        默认返回当前监听端口所有转发规则列表；
        指定ID返回指定规则字典；
        """
        if rule_id == 'all':
            request = DescribeRulesRequest.DescribeRulesRequest()
            request.set_accept_format('json')
            request.set_LoadBalancerId(slb_id)
            request.set_ListenerPort(listener_port)
            result = self.client.do_action(request)['Rules']['Rule']
            return result
        request = DescribeRuleAttributeRequest.DescribeRuleAttributeRequest()
        request.set_accept_format('json')
        request.set_RuleId(rule_id)
        result = self.client.do_action(request)
        return result

    def get_vgroups(self, slb_id):
        """
        :params: SLB ID
        :return: 返回当前SLB实例虚拟组列表
        """
        request = DescribeVServerGroupsRequest \
            .DescribeVServerGroupsRequest()
        request.set_accept_format('json')
        request.set_LoadBalancerId(slb_id)
        result = self.client.do_action(request)['VServerGroups']['VServerGroup']
        return result

    def get_vgroups_attributes(self, slb_id):
        """
        :params: SLB ID
        :return: 返回指定SLB下虚拟组详情列表
        """
        vgroup_attr_list = []
        slb_vgroups = self.get_vgroups(slb_id)
        request = DescribeVServerGroupAttributeRequest. \
            DescribeVServerGroupAttributeRequest()
        request.set_accept_format('json')
        for vgroup in slb_vgroups:
            request.set_VServerGroupId(vgroup['VServerGroupId'])
            result = self.client.do_action(request)
            result.pop('RequestId')
            vgroup_attr_list.append(result)
        return vgroup_attr_list

    def create_slb(self, **kwargs):
        request = CreateLoadBalancerRequest.CreateLoadBalancerRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_slb_mini(self, **kwargs):
        params = {
            'Bandwidth': 1,
            'InternetChargeType': 'paybytraffic',
            'LoadBalancerSpec': 'slb.s1.small',
            'PayType': 'PayOnDemand'
        }
        kwargs.update(params)
        # pylint: disable=E1121
        result = self.create_slb(**kwargs)
        return result

    def set_tcp_listener_attr(self, **kwargs):
        """
        设置SLB TCP监听属性
        https://help.aliyun.com/document_detail/27604.html?spm=a2c4g.11186623.6.645.7e0de5afZXm5kS
        """
        request = SetLoadBalancerTCPListenerAttributeRequest\
            .SetLoadBalancerTCPListenerAttributeRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result



from .. import base  # pylint: disable=unuse-import


class BaseAliyunApi:
    """ Aliyun base class"""

    def __init__(self, client=None):
        self.client = client  # type: base.BaseAliyunClient



#-*- coding:utf-8 -*-
from aliyunsdkvpc.request.v20160428 import DescribeVpcsRequest, \
        DescribeVSwitchesRequest, \
        DescribeVRoutersRequest, DescribeRouteTablesRequest, \
        DescribeNatGatewaysRequest, DescribeSnatTableEntriesRequest, \
        CreateVpcRequest, CreateVSwitchRequest, \
        CreateNatGatewayRequest, CreateBandwidthPackageRequest, \
        CreateSnatEntryRequest
from .base import BaseAliyunApi


class AliyunVpc(BaseAliyunApi):
    """
    阿里云VPC相关API：
        VPC,
        Vswitch(虚拟交换机),
        VRouter(虚拟路由器),
        VRouter Table(路由表),
        NAT Gateway, bandwidth package, SNAT
        相关资源信息获取及创建；
    """

    def get_vpc_list(self):
        """
        :return: 当前可用区下VPC列表
        """
        request = DescribeVpcsRequest.DescribeVpcsRequest()
        request.set_accept_format('json')
        result = self.client.do_action(request)['Vpcs']['Vpc']
        return result

    def get_vswitch_list(self, vpc_id):
        """
        :params: VPC实例ID
        :return: 指定VPC下虚拟交换机Vswitch列表
        默认页显示50；
        """
        request = DescribeVSwitchesRequest.DescribeVSwitchesRequest()
        request.set_accept_format('json')
        request.set_PageSize(50)
        request.set_VpcId(vpc_id)
        result = self.client.do_action(request)['VSwitches']['VSwitch']
        return result

    def get_vrouter_list(self, vrouter_id='all'):
        """
        :params: Vrouter虚拟路由器ID
        :return: 默认返回路由器列表
                 指定id返回对应路由器字典；
        默认页显示50；
        """
        request = DescribeVRoutersRequest.DescribeVRoutersRequest()
        request.set_accept_format('json')
        request.set_PageSize(50)
        if vrouter_id == 'all':
            return self.client.do_action(request)['VRouters']['VRouter']
        request.set_VRouterId(vrouter_id)
        return self.client.do_action(request)['VRouters']['VRouter'][0]

    def get_vroute_table(self, vrouter_id):
        """
        :params: Vrouter虚拟路由器ID
        :return: 路由表规则列表
        默认页显示50；
        """
        request = DescribeRouteTablesRequest.DescribeRouteTablesRequest()
        request.set_accept_format('json')
        request.set_PageSize(50)
        request.set_VRouterId(vrouter_id)
        return self.client.do_action(request)['RouteTables']['RouteTable']

    def get_nat_gateways(self, nat_gateway_id='all'):
        """
        :params: Nat网关ID
        :return: 默认Nat网关列表 指定ID返回对应网关信息
        默认也显示50；
        """
        request = DescribeNatGatewaysRequest.DescribeNatGatewaysRequest()
        request.set_accept_format('json')
        request.set_PageSize(50)
        if nat_gateway_id == 'all':
            return self.client.do_action(request)['NatGateways']['NatGateway']
        request.set_NatGatewayId(nat_gateway_id)
        return self.client.do_action(request)['NatGateways']['NatGateway'][0]

    def get_nat_gateway_eiplist(self, nat_gateway_id='all'):
        """
        :params: Nat网关ID
        :return: 返回NAT网关IP列表
        """
        nat_gateway = self.get_nat_gateways(nat_gateway_id)
        if isinstance(nat_gateway, list):
            result = [ip['IpAddress'] for nat in nat_gateway \
                    for ip in nat['IpLists']['IpList']]
        elif isinstance(nat_gateway, dict):
            result = [ip['IpAddress'] for ip in nat_gateway['IpLists']['IpList']]
        else:
            result = None
        return result

    def get_nat_bandwidthlist(self, nat_gateway_id='all'):
        """
        :params: Nat网关ID
        :return: 返回NAT网关带宽包ID列表
        """
        nat_gateway = self.get_nat_gateways(nat_gateway_id)
        if isinstance(nat_gateway, list):
            bandwidthlist = []
            for nat in nat_gateway:
                bandwidthlist.extend(nat['BandwidthPackageIds']['BandwidthPackageId'])
            result = bandwidthlist
        elif isinstance(nat_gateway, dict):
            result = nat_gateway['BandwidthPackageIds']['BandwidthPackageId']
        else:
            result = None
        return result

    def get_snat_table_ids(self, nat_gateway_id='all'):
        """
        :params: NAT网关ID
        :return: SNAT表ID列表
        """
        nat_gateway = self.get_nat_gateways(nat_gateway_id)
        if isinstance(nat_gateway, list):
            snat_table_idlist = []
            for nat in nat_gateway:
                snat_table_idlist.extend(nat['SnatTableIds']['SnatTableId'])
            snat_table_ids = snat_table_idlist
        elif isinstance(nat_gateway, dict):
            snat_table_ids = nat_gateway['SnatTableIds']['SnatTableId']
        else:
            snat_table_ids = None
        return snat_table_ids

    def get_snat_table_entries(self, snat_table_id):
        """
        :params: SNAT表ID
        :return: 返回SNAT表条目列表
        """
        request = DescribeSnatTableEntriesRequest\
                .DescribeSnatTableEntriesRequest()
        request.set_accept_format('json')
        request.set_PageSize(50)
        request.set_SnatTableId(snat_table_id)
        result = self.client\
                .do_action(request)['SnatTableEntries']['SnatTableEntry']
        return result

    # pylint: disable=C0303 
    def create_vpc(self, **kwargs):
        """
        https://help.aliyun.com/document_detail/\
                35737.html?spm=a2c4g.11174283.6.603.2cc96dd4NCrbSa
        :params:
            {
                'RegionId': '可用区ID',
                'VpcName': 'Vpc名',
                'CidrBlock': [
                    '10.0.0.0/8',
                    '172.16.0.0/12',
                    '192.168.0.0/16'
                    ],
                'EnableIpv6': False,
                'Ipv6CidrBlock': '223.223.213.122',
                'Description': '描述',
            }
        :return:
            VPC ID, VRouter ID, RouterTable ID
        """
        request = CreateVpcRequest.CreateVpcRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_vswitch(self, **kwargs):
        """
        :params:
            {
                'VpcId': 'VPC ID',
                'RegionId': '可用区',
                'ZoneId': '所属区ID',
                'VSwitchName': '交换机名字',
                'CidrBlock': 'IPv4网段',
                'Ipv6CidrBlock': 'IPv6网段',
                'Description': '描述',
            }
        """
        request = CreateVSwitchRequest.CreateVSwitchRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_nat_gateway(self, **kwargs):
        """
        :params:
            {
                'VpcId': 'VPC ID',
                'RegionId': '可用区ID',
                'Name': '网关名',
                'Spec': [
                    'Small',
                    'Middle',
                    'Large',
                    'XLarge'
                    ],
                'Description': '描述',
            }
        :return:
            NatGatewayId, ForwardTableIds,
            SnatTableIds, SnatTableIds
        """
        request = CreateNatGatewayRequest.CreateNatGatewayRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_bandwidth_pkg(self, **kwargs):
        """
        :params:
            {
                'NatGatewayId': 'NAT网关ID',
                'RegionId': '可用区ID',
                'Name': '带宽包名',
                'Bandwidth': '5',
                'IpCount': '1,2',
                'InternetChargeType': 'PayByTraffic',
                'Zone': 'cn-qingdao-a',
                'Description': '描述',
            }
        """
        request = CreateBandwidthPackageRequest\
                .CreateBandwidthPackageRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_snat_entry(self, **kwargs):
        """
        :params:
            {
                'RegionId': '可用区ID',
                'SnatIp': 'SNAT IP',
                'SnatTableId': 'SNAT表ID',
                'SourceVSwitchId': '包含Vswitch ID',
            }
        """
        request = CreateSnatEntryRequest.CreateSnatEntryRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result



from aliyunsdkdrds.request.v20171016 import (
    DescribeDrdsInstancesRequest, DescribeRegionsRequest
)
from .base import BaseAliyunApi


class AliyunDrds(BaseAliyunApi):
    def get_regions(self, id_only=False):
        """
        获取支持的region列表
        https://help.aliyun.com/document_detail/51129.html?spm=a2c4g.11186623.6.777.3af07cbdrI2FFq
        :param id_only: 是否只返回region id列表
        :return:
        """
        request = DescribeRegionsRequest.DescribeRegionsRequest()
        result = self.client.do_action(request)
        result_list = result.get('DrdsRegions', {}).get('DrdsRegion', [])
        if id_only:
            result_list = list(set([x.get('RegionId') for x in result_list]))
        return result_list

    def list_drds(self, name_only=False, **kwargs):
        """
        获取实例列表信息
        https://help.aliyun.com/document_detail/51128.html?spm=a2c4g.11186623.6.776.263a6fefn8XahW
        :param name_only: 是否只返回实例名列表
        :param kwargs:
        :return:
        """
        result_list = []
        request = DescribeDrdsInstancesRequest.DescribeDrdsInstancesRequest()
        regions = self.get_regions(id_only=True)
        for region in regions:
            for k, v in kwargs.items():
                request.add_query_param(k, v)
            request.add_query_param('RegionId', region)
            result = self.client.do_action(request)
            result_list.extend(result.get('Data', {}).get('Instance', []))

        if name_only:
            result_list = [i['InstanceName'] for i in result_list]
        return result_list



class AliyunException(Exception):
    def __init__(self, errcode=None, errmsg=None):
        self.errcode = errcode
        self.errmsg = errmsg

    def __str__(self):
        return 'Error code: {0}, message: {1}'.format(self.errcode, self.errmsg)

    def __repr__(self):
        return '{cls}({code}, {msg})'.format(cls=self.__class__.__name__,\
            code=self.errcode, msg=self.errmsg)


class AliyunClientException(AliyunException):
    pass



import time
from aliyunsdkons.request.v20190214 import (
    OnsRegionListRequest, OnsInstanceInServiceListRequest,
    OnsTopicListRequest
)
from .base import BaseAliyunApi


class AliyunOns(BaseAliyunApi):
    def get_regions(self, id_only=False):
        """
        获取支持的region列表
        https://help.aliyun.com/document_detail/29622.html?spm=a2c4g.11186623.6.594.66a64614BsQKrj
        :param id_only: 是否只返回region id列表
        :return:
        """
        request = OnsRegionListRequest.OnsRegionListRequest()
        request.set_PreventCache(int(time.time()))
        result = self.client.do_action(request)
        result_list = result.get('Data', {}).get('RegionDo', [])
        if id_only:
            result_list = list(set([x.get('OnsRegionId') for x in result_list]))
        return result_list

    def get_instances(self):
        """
        获取实例列表, 返回所有region
        https://help.aliyun.com/document_detail/106351.html?spm=a2c4g.11186623.6.599.22155c8c7rg3Wk

        {
          "Data": {
            "InstanceVO": [
              {
                "InstanceStatus": 5,
                "IndependentNaming": false,
                "InstanceId": "MQ_INST_1726708279589269_C54T0Yxx",
                "InstanceName": "DEFAULT_INSTANCE",
                "InstanceType": 1
              }
            ]
          },
          "HelpUrl": "",
          "RequestId": "CCD70385-9445-42F1-959E-A4EC62756DE5"
        }
        :return:
        {
          "cn-beijing": [
            {
              "InstanceStatus": 5,
              "IndependentNaming": false,
              "InstanceId": "MQ_INST_1726708279589269_C54T0Yxx",
              "InstanceName": "DEFAULT_INSTANCE",
              "InstanceType": 1
            }
          ]
        }
        """
        result_dict = {}
        request = OnsInstanceInServiceListRequest.OnsInstanceInServiceListRequest()
        request.set_PreventCache(int(time.time()))
        regions = self.get_regions(id_only=True)
        for region in regions:
            self.client.acs_client.set_region_id(region)
            result = self.client.do_action(request)
            instance_list = result.get('Data', {}).get('InstanceVO', [])
            if instance_list:
                result_dict[region] = instance_list
        return result_dict

    def list_topic(self, region='mq-internet-access', instance_id=None):
        """
        查询账号下所有 Topic 的信息列表
        https://help.aliyun.com/document_detail/29590.html?spm=a2c4g.11174283.6.604.31de449cQdtRnc
        :param instance_id: ons 实例id
        :param region: region id
        :return:
         [{
            "Relation": 1,
            "Owner": "1726708279589269",
            "RelationName": "所有者",
            "IndependentNaming": false,
            "InstanceId": "MQ_INST_1726708279589269_yyyyy8Ak",
            "CreateTime": 1446036003000,
            "Topic": "TeleSpiderTest",
            "Remark": "",
            "MessageType": 0
          }
        ]
        """
        request = OnsTopicListRequest.OnsTopicListRequest()
        request.set_PreventCache(int(time.time()))
        request.set_InstanceId(instance_id)
        self.client.acs_client.set_region_id(region)
        result = self.client.do_action(request)
        topics = result.get('Data', {}).get('PublishInfoDo', [])
        return topics

    def list_topic_all(self):
        """
        获取所有region和instance下的topic
        :return:
        [{
          "RegionId": "ap-southeast-1",
          "Relation": 1,
          "Owner": "1726708279589269",
          "RelationName": "所有者",
          "IndependentNaming": false,
          "InstanceId": "MQ_INST_1726708279589269_yyyyy6d0",
          "CreateTime": 1546401123000,
          "Topic": "MarsEventTaskSingapore",
          "Remark": "新加坡集群风控结果通知",
          "MessageType": 0
        }]
        """
        result_list = []
        instance_dict = self.get_instances()
        for region, instances in instance_dict.items():
            for instance in instances:
                topics = self.list_topic(region=region,
                                         instance_id=instance.get('InstanceId'))
                for topic in topics:
                    result_list.append(dict(RegionId=region, **topic))
        return result_list



import copy
from aliyunsdkpvtz.request.v20180101 import (
    DescribeZonesRequest, DescribeZoneRecordsRequest
)
from .base import BaseAliyunApi


class AliyunDnsPrivateZone(BaseAliyunApi):
    def get_zones(self):
        """
        查询zone列表
        https://help.aliyun.com/document_detail/66243.html?spm=a2c4g.11186623.6.569.24fc6cc6L0G6Qs
        :return:
        """
        client = copy.deepcopy(self.client)
        client.acs_client.set_region_id('cn-hangzhou')
        request = DescribeZonesRequest.DescribeZonesRequest()
        request.set_PageSize(100)
        result = client.do_action(request)
        return result.get('Zones', {}).get('Zone', [])

    def list_zone_records(self, zoneid=None, **kwargs):
        """
        查询解析记录
        https://help.aliyun.com/document_detail/66252.html?spm=a2c4g.11186623.6.580.61a174bbMO2TSv
        :param zoneid: domain zone id, 不填遍历所有
        :param kwargs: 扩展参数
        :return:
        {
          "xxxx.com": [
            {
              "Status": "ENABLE",
              "Value": "127.0.0.1",
              "Rr": "private-test",
              "RecordId": 129807,
              "Ttl": 60,
              "Type": "A"
            }
          ]
        }
        """
        client = copy.deepcopy(self.client)
        client.acs_client.set_region_id('cn-hangzhou')

        result_dict = {}
        request = DescribeZoneRecordsRequest.DescribeZoneRecordsRequest()
        request.set_PageSize(100)
        request.set_PageNumber(1)
        for k, v in kwargs.items():
            request.add_query_param(k, v)

        zone_list = [zoneid] if zoneid else \
            [(x.get('ZoneId'), x.get('ZoneName')) for x in self.get_zones()]

        for x in zone_list:
            result_list = []
            zone_id, zone_name = x
            request.set_ZoneId(zone_id)
            result = client.do_action(request)
            total_count = float(result.get('TotalItems', 0))
            result_list.extend(result.get('Records').get('Record'))
            if not total_count:
                result_dict[zone_name] = result_list
                continue
            page_count = float(result.get('PageSize', 1))
            _p = total_count / page_count
            if _p > 1:
                for _p_num in range(2, int(_p) + 2):
                    request.set_PageNumber(_p_num)
                    result = client.do_action(request)
                    result_list.extend(result.get('ReCords').get('ReCord'))
            result_dict[zone_name] = result_list
        return result_dict



import string
import time
from random import choice
from aliyunsdkr_kvstore.request.v20150101 import (
    CreateInstanceRequest, DescribeRegionsRequest,
    DescribeInstancesRequest
)
from .base import BaseAliyunApi


class AliyunRedis(BaseAliyunApi):
    def get_regions(self, id_only=False):
        """
        获取支持的region列表
        https://help.aliyun.com/document_detail/61012.html?spm=a2c4g.11186623.6.687.2b10ae6dPxXy2S
        :param id_only: 是否只返回region id列表
        :return:
        """
        request = DescribeRegionsRequest.DescribeRegionsRequest()
        result = self.client.do_action(request)
        result_list = result.get('RegionIds', {}).get('KVStoreRegion', [])
        if id_only:
            result_list = list(set([x.get('RegionId') for x in result_list]))
        return result_list

    def list_redis(self, name_only=False, **kwargs):
        """
        获取实例列表信息
        https://help.aliyun.com/document_detail/60933.html?spm=a2c4g.11186623.6.694.67002cacaswc5I
        :param name_only:
        :param kwargs:
        :return:
        """
        result_list = []
        request = DescribeInstancesRequest.DescribeInstancesRequest()
        regions = self.get_regions(id_only=True)
        for region in regions:
            request.set_PageSize(100)
            request.set_PageNumber(1)
            for k, v in kwargs.items():
                request.add_query_param(k, v)
            request.add_query_param('RegionId', region)
            result = self.client.do_action(request)
            result_list.extend(result.get('Instances', {}).get('KVStoreInstance', []))

            total_count = float(result.get('TotalCount', 0))
            if not total_count:
                continue
            page_count = float(result.get('PageSize', 1))
            _p = total_count / page_count
            if _p > 1:
                for _p_num in range(2, int(_p) + 2):
                    request.set_PageNumber(_p_num)
                    result = self.client.do_action(request)
                    result_list.extend(result.get('Instances', {}).get('KVStoreInstance', []))
            # 防止被限流
            time.sleep(1)
        if name_only:
            result_list = [i['InstanceName'] for i in result_list]
        return result_list

    def create_redis_instance(self, **kwargs):
        request = CreateInstanceRequest.CreateInstanceRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_redis_instance_mini(self, **kwargs):
        """
        :return: result && password
        """
        base_str = string.digits + string.ascii_letters
        _password = ''.join([choice(base_str) \
                for _ in range(30)])
        params = {
            #'InstanceClass': 'redis.logic.splitrw.small.1db.3rodb.4proxy.default',
            'InstanceClass': 'redis.master.small.default',
            'Password': _password,
            'NetworkType': 'VPC',
            'InstanceType': 'Redis',
            'ChargeType': 'PrePaid',
            'Period': 1,
            'EngineVersion': '2.8',
            }
        kwargs.update(params)
        result = self.create_redis_instance(**kwargs)
        result.update({'Password': _password})
        return result



# -*- coding:utf-8 -*-
from aliyunsdkalidns.request.v20150109 import DescribeDomainsRequest, \
    DescribeDomainRecordsRequest
from .base import BaseAliyunApi


class AliyunDns(BaseAliyunApi):
    """
    阿里云DNS相关API
    """

    def get_dns_list(self):
        """
        获取DNS列表
        :return: DNS列表
        """
        request = DescribeDomainsRequest.DescribeDomainsRequest()
        request.set_accept_format('json')
        request.set_PageNumber(1)
        request.set_PageSize(100)

        first_page_result = self.client.do_action(request)
        total_count, page_size = first_page_result['TotalCount'], \
                                 first_page_result['PageSize']

        if total_count <= page_size:
            return first_page_result['Domains']['Domain']
        result_list = first_page_result['Domains']['Domain']
        _page_num = (total_count / page_size) \
            if total_count % page_size == 0 \
            else (int(total_count / page_size) + 1)
        for _page in range(2, _page_num + 1):
            request.set_PageNumber(_page)
            page_result = self.client.do_action(request)
            result_list.extend(page_result['Domains']['Domain'])
        return result_list

    def get_record_count(self, domain_name):
        """
        :return: 对应域名解析记录总数
        """
        request = DescribeDomainRecordsRequest.DescribeDomainRecordsRequest()
        request.set_accept_format('json')
        request.set_DomainName(domain_name)
        record_count = self.client.do_action(request)['TotalCount']
        return record_count

    def get_dns_record(self, domain_name):
        """
        获取DNS解析记录
        :params: 二级域名
        :return: 对应域名解析记录
        """
        request = DescribeDomainRecordsRequest.DescribeDomainRecordsRequest()
        request.set_accept_format('json')
        request.set_PageNumber(1)
        request.set_PageSize(100)
        request.set_DomainName(domain_name)

        first_page_result = self.client.do_action(request)
        total_count, page_size = first_page_result['TotalCount'], \
                                 first_page_result['PageSize']

        if total_count <= page_size:
            return first_page_result['DomainRecords']['Record']
        result_list = first_page_result['DomainRecords']['Record']
        _page_num = (total_count / page_size) \
            if total_count % page_size == 0 \
            else (int(total_count / page_size) + 1)
        for _page in range(2, _page_num + 1):
            request.set_PageNumber(_page)
            page_result = self.client.do_action(request)
            result_list.extend(page_result['DomainRecords']['Record'])
        return result_list



import json
import logging
import datetime


def get_logger(name=None):
    return logging.getLogger('aliyunpy.%s' % name)


def date_to_isoformat_datetime(d, timedelta=8):
    return (d - datetime.timedelta(hours=timedelta)).isoformat(timespec='seconds') + 'Z'


def isoformat_to_datetime(s, timedelta=8, strp='%Y-%m-%dT%H:%M:%SZ'):
    return datetime.datetime.strptime(s, strp) + datetime.timedelta(hours=timedelta)


class MyJsonEncoder(json.JSONEncoder):
    def default(self, o):
        if hasattr(o, '__dict__'):
            return {k.lstrip('_'): v for k, v in o.__dict__.items()}
        else:
            return o.__str__()



# pylint: disable=all
import json
from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.http import format_type
from aliyunsdkcore.http.http_response import HttpResponse


def patch_all():
    AcsClient._make_http_response = _make_http_response


def _make_http_response(self, endpoint, request, timeout, specific_signer=None):
    body_params = request.get_body_params()
    if body_params:
        body = json.dumps(body_params)
        request.set_content(body)
        request.set_content_type(format_type.APPLICATION_JSON)
    elif request.get_content() and "Content-Type" not in request.get_headers():
        request.set_content_type(format_type.APPLICATION_OCTET_STREAM)
    method = request.get_method()

    signer = self._signer if specific_signer is None else specific_signer
    header, url = signer.sign(self._region_id, request)

    if self.get_user_agent() is not None:
        header['User-Agent'] = self.get_user_agent()
    if header is None:
        header = {}
    header['x-sdk-client'] = 'python/2.0.0'

    protocol = request.get_protocol_type()
    response = HttpResponse(
        endpoint,
        url,
        method,
        header,
        protocol,
        request.get_content(),
        self._port,
        timeout=timeout)
    if body_params:
        body = json.dumps(body_params)
        response.set_content(body, "utf-8", format_type.APPLICATION_JSON)
    return response



import random
import string
import time

from aliyunsdkrds.request.v20140815 import CreateDBInstanceRequest, \
    CreateDatabaseRequest, DescribeRegionsRequest, \
    CreateAccountRequest, GrantAccountPrivilegeRequest, \
    DescribeDBInstancesRequest
from .base import BaseAliyunApi


class AliyunRds(BaseAliyunApi):
    """
    Aliyun RDS related API
    """

    def get_regions(self, id_only=False):
        """
        获取支持的region列表
        https://help.aliyun.com/document_detail/26243.html
        :param id_only: 是否只返回region id列表
        :return:
        """
        request = DescribeRegionsRequest.DescribeRegionsRequest()
        result = self.client.do_action(request)
        result_list = result.get('Regions', {}).get('RDSRegion', [])
        if id_only:
            result_list = list(set([x.get('RegionId') for x in result_list]))
        return result_list

    def list_rds(self, name_only=False, **kwargs):
        """
        获取实例列表
        https://help.aliyun.com/document_detail/26232.html?spm=a2c4g.11186623.6.1322.16ff69b2We0YoY
        :param name_only: 是否只返回name列表
        :param kwargs:
        :return:
        """
        result_list = []
        request = DescribeDBInstancesRequest.DescribeDBInstancesRequest()
        regions = self.get_regions(id_only=True)
        for region in regions:
            request.set_PageSize(100)
            request.set_PageNumber(1)
            for k, v in kwargs.items():
                request.add_query_param(k, v)
            request.add_query_param('RegionId', region)
            result = self.client.do_action(request)
            result_list.extend(result.get('Items', {}).get('DBInstance', []))

            total_count = float(result.get('TotalCount', 0))
            if not total_count:
                continue
            page_count = float(result.get('PageSize', 1))
            _p = total_count / page_count
            if _p > 1:
                for _p_num in range(2, int(_p) + 2):
                    # 防止限流
                    time.sleep(1)
                    request.set_PageNumber(_p_num)
                    result = self.client.do_action(request)
                    result_list.extend(result.get('Items', {}).get('DBInstance', []))

        if name_only:
            result_list = [i['InstanceName'] for i in result_list]
        return result_list

    def create_rds(self, **kwargs):
        request = CreateDBInstanceRequest.CreateDBInstanceRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_rds_mini(self, **kwargs):
        """
        RDS最小化配置
        """
        params = {
            'Engine': 'MySQL',
            'EngineVersion': '5.6',
            'DBInstanceClass': 'rds.mysql.s2.large',
            'DBInstanceStorage': 100,
            'DBInstanceNetType': 'Intranet',
            'DBInstanceStorageType': 'local_ssd',
            'SystemDBCharset': 'utf8',
            'InstanceNetworkType': 'VPC',
            # 'VPCId': 'VPC ID',
            # 'VSwitchId': 'Vswitch ID',
            'PayType': 'Prepaid',
            'UsedTime': 1,
            'Period': 'Month',
        }
        kwargs.update(params)
        result = self.create_rds(**kwargs)
        return result

    def create_rds_database(self, **kwargs):
        request = CreateDatabaseRequest.CreateDatabaseRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_rds_databases(self, instance_id, db_list, charset='utf8'):
        results = []
        if not isinstance(db_list, list):
            raise TypeError
        for db in db_list:
            result = self.create_rds_database(
                DBInstanceId=instance_id,
                DBName=db,
                CharacterSetName=charset,
            )
            results.append(result)
        return results

    def create_rds_account(self, **kwargs):
        request = CreateAccountRequest.CreateAccountRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def grant_account_privi(self, **kwargs):
        request = GrantAccountPrivilegeRequest.GrantAccountPrivilegeRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_database_apollo(self, instance_id):
        account = 'apollo'
        db_list = ['configserverprod', 'portalserver']
        _password_base = string.ascii_letters + \
                         string.digits + \
                         '!@#$%^&'
        _password_length = 16
        _password = ''.join([random.choice(_password_base) for _ in \
                             range(_password_length)])
        # Create databases
        db_result = self.create_rds_databases(
            instance_id,
            db_list,
            charset='utf8',
        )
        if not db_result:
            raise Exception("Create Apollo databases failed.")
        # Create apollo account
        account_result = self.create_rds_account(
            DBInstanceId=instance_id,
            AccountName=account,
            AccountPassword=_password,
            AccountDescription='Apollo account',
        )
        if not account_result:
            raise Exception("Create Apollo account failed.")
        # Grant privileges to apollo
        for db in db_list:
            grant_result = self.grant_account_privi(
                DBInstanceId=instance_id,
                AccountName=account,
                DBName=db,
                AccountPrivilege='ReadWrite',
            )
            if not grant_result:
                raise Exception("Grant to apollo failed.")
        return {'Account': 'apollo', 'Password': _password}



from aliyunsdkecs.request.v20140526 import DescribeInstancesRequest, \
    CreateInstanceRequest, StartInstanceRequest, \
    RebootInstanceRequest, \
    DescribeEipAddressesRequest, \
    AllocateEipAddressRequest, AllocatePublicIpAddressRequest, \
    AssociateEipAddressRequest, UnassociateEipAddressRequest, \
    ModifyInstanceAttributeRequest, DescribeRegionsRequest
from .base import BaseAliyunApi


# pylint: disable=E1121


class AliyunEcs(BaseAliyunApi):
    def get_regions(self, id_only=False):
        """
        获取支持的region列表
        https://help.aliyun.com/document_detail/25609.html
        :param id_only: 是否只返回region id列表
        :return:
        """
        request = DescribeRegionsRequest.DescribeRegionsRequest()
        result = self.client.do_action(request)
        result_list = result.get('Regions', {}).get('Region', [])
        if id_only:
            # pylint: disable=R1718
            result_list = list(set([x.get('RegionId') for x in result_list]))
        return result_list

    def list_ecs(self, name_only=False, **kwargs):
        """
        获取实例列表信息
        https://help.aliyun.com/document_detail/25506.html?spm=a2c4g.11186623.6.941.2b453215sQTZIS
        :param name_only:
        :param kwargs:
        :return:
        """
        result_list = []
        request = DescribeInstancesRequest.DescribeInstancesRequest()
        regions = self.get_regions(id_only=True)
        for region in regions:
            request.set_PageSize(100)
            request.set_PageNumber(1)
            for k, v in kwargs.items():
                request.add_query_param(k, v)
            request.add_query_param('RegionId', region)
            result = self.client.do_action(request)
            result_list.extend(result.get('Instances', {}).get('Instance'))

            total_count = float(result.get('TotalCount', 0))
            if not total_count:
                continue
            page_count = float(result.get('PageSize', 1))
            _p = total_count / page_count
            if _p > 1:
                for _p_num in range(2, int(_p) + 2):
                    request.set_PageNumber(_p_num)
                    result = self.client.do_action(request)
                    result_list.extend(result.get('Instances').get('Instance'))

        if name_only:
            result_list = [i['InstanceName'] for i in result_list]
        return result_list

    def get_ecs_proxy(self, name_only=False, **kwargs):
        """
        :return: ecs instance with tag: instance_category=proxy
        """
        proxy_list = []
        proxy_tag = {'TagKey': 'instance_category', 'TagValue': 'proxy'}
        result_list = self.list_ecs(**kwargs)
        for result in result_list:
            if 'Tags' in result.keys():
                if proxy_tag in result['Tags']['Tag']:
                    if name_only:
                        proxy_list.append(result['InstanceName'])
                    else:
                        proxy_list.append(result)
        return proxy_list

    def create_ecs(self, **kwargs):
        """
        :return: ECS创建实例ID
        """
        request = CreateInstanceRequest.CreateInstanceRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def restart_ecs(self, **kwargs):
        """
        : 重启ECS实例
        """
        request = RebootInstanceRequest.RebootInstanceRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def start_ecs(self, **kwargs):
        """
        :params: ECS实例ID
        """
        request = StartInstanceRequest.StartInstanceRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def start_batch_ecs(self, instance_ids, **kwargs):
        """
        :params: Instance ID列表
        """
        results = []
        for instance_id in instance_ids:
            result = self.start_ecs(InstanceId=instance_id, **kwargs)
            results.append(result)
        return results

    def create_ecs_proxy(self, **kwargs):
        """
        :params: 爬虫交换机ID
        """
        params = {
            'InstanceType': 'ecs.t5-lc2m1.nano',
            'IoOptimized': 'optimized',
            'ImageId': 'ubuntu_16_0402_64_20G_alibase_20171227.vhd',
            'KeyPairName': 'admin',
            'Tag.1.Key': 'instance_category',
            'Tag.1.Value': 'proxy',
            'InstanceChargeType': 'PrePaid',
            'Period': 1,
            'PeriodUnit': 'Month',
            'AutoRenew': True,
            'InternetMaxBandwidthOut': 1,
            'AutoRenewPeriod': 1
        }
        kwargs.update(params)
        result = self.create_ecs(**kwargs)
        return result

    def create_ecs_proxys(self, num=1, **kwargs):
        """
        :params: proxy ECS数量
        """
        results = []
        for _ in range(num):
            result = self.create_ecs_proxy(**kwargs)
            results.append(result)
        return results

    def create_ecs_mini(self, **kwargs):
        """
        :params: 交换机ID
        """
        params = {
            'InstanceType': 'ecs.g5.xlarge',
            'IoOptimized': 'optimized',
            'ImageId': 'ubuntu_16_0402_64_20G_alibase_20171227.vhd',
            'KeyPairName': 'admin',
            'InstanceChargeType': 'PrePaid',
            'Period': 1,
            'PeriodUnit': 'Month',
            'AutoRenew': True,
            'AutoRenewPeriod': 1
        }
        kwargs.update(params)
        result = self.create_ecs(**kwargs)
        if 'InstanceId' not in result.keys():
            raise KeyError('InstanceId')
        start_result = self.start_ecs(InstanceId=result['InstanceId'])
        return start_result

    def list_eip(self, **kwargs):
        result_list = []
        request = DescribeEipAddressesRequest.DescribeEipAddressesRequest()
        request.set_PageSize(50)
        regions = self.get_regions(id_only=True)
        for region in regions:
            request.set_PageNumber(1)
            for k, v in kwargs.items():
                request.add_query_param(k, v)
            request.add_query_param('RegionId', region)
            result = self.client.do_action(request)
            result_list.extend(result.get(\
                'EipAddresses', {}).get(\
                'EipAddress', []))
            total_count = float(result.get('TotalCount', 0))
            if not total_count:
                continue
            page_count = float(result.get('PageSize', 1))
            _p = total_count / page_count
            if _p > 1:
                for _p_num in range(2, int(_p) + 2):
                    request.set_PageNumber(_p_num)
                    result = self.client.do_action(request)
                    result_list.extend(result.get(\
                        'EipAddresses', {}).get(\
                        'EipAddress', []))
        return result_list

    def create_eip(self, **kwargs):
        """
        :return: 创建EIP及ID
        """
        request = AllocateEipAddressRequest.AllocateEipAddressRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_batch_eip(self, num=1, **kwargs):
        """
        :return: 创建EIP及ID列表
        """
        results = []
        params = {
            'Bandwidth': 5,
            'InternetChargeType': 'PayByTraffic',
        }
        kwargs.update(params)
        for _ in range(num):
            result = self.create_eip(**kwargs)
            results.append(result)
        return results

    def associate_eip(self, instance_id, eip_id, instance_type='Ecs'):
        """
        :params: ECS ID, EIP IP
        :return: 绑定结果
        """
        if instance_type not in ['Nat', 'Slb', 'Ecs']:
            raise TypeError('Wrong instance type.')
        request = AssociateEipAddressRequest.AssociateEipAddressRequest()
        request.set_InstanceType(instance_type)
        request.set_InstanceId(instance_id)
        request.set_AllocationId(eip_id)
        result = self.client.do_action(request)
        return result

    def associate_ecs_eip(self, ecs_id, eip_id):
        """
        :params: ECS实例ID & EIP ID
        """
        result = self.associate_eip(ecs_id, eip_id)
        return result

    def associate_nat_eip(self, nat_gateway_id, eip_id):
        """
        :params: NAT网关ID & EIP ID
        """
        result = self.associate_eip(nat_gateway_id, \
                                    eip_id, instance_type='Nat')
        return result

    def unassociate_ecs_eip(self, ecs_id, eip_id):
        """
        :return: EIP解绑结果
        """
        request = UnassociateEipAddressRequest.UnassociateEipAddressRequest()
        request.set_InstanceType("EcsInstance")
        request.set_InstanceId(ecs_id)
        request.set_AllocationId(eip_id)
        result = self.client.do_action(request)
        return result

    def modify_instance_attr(self, instance_id=None, **kwargs):
        """
        修改实例信息
        https://help.aliyun.com/document_detail/25503.html?spm=a2c4g.11174283.6.950.3c5b52feWdhQnd

        :param instance_id:
        :param kwargs:
        """
        request = ModifyInstanceAttributeRequest.ModifyInstanceAttributeRequest()
        request.set_InstanceId(instance_id)
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def modify_instance_name(self, instance_ids, name_prefix, start_index=1):
        results = []
        for i in instance_ids:
            _index = start_index + instance_ids.index(i)
            _index_id = str(_index) if _index > 9 else "0{}".format(_index)
            instance_name = "{}{}".format(name_prefix, _index_id)
            result = self.modify_instance_attr(
                i,
                InstanceName=instance_name,
                HostName=instance_name)
            results.append(result)
        return results

    def allocate_pub_ip(self, **kwargs):
        """
        :params: ECS实例ID
        """
        request = AllocatePublicIpAddressRequest.AllocatePublicIpAddressRequest()
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result



from aliyunsdkcore.request import RpcRequest
from .base import BaseAliyunApi


class DescribeDomainNamesRequest(RpcRequest):
    """
    WAF API: DescribeDomainNames
    Write due to lack of Aliyun WAF SDK
    """
    def __init__(self):
        RpcRequest.__init__(self, 'Waf', '2018-01-17', \
            'DescribeDomainNames', 'waf', protocol='https')

    def get_InstanceId(self):
        return self.get_query_params().get('InstanceId')

    def set_InstanceId(self, InstanceId):
        self.add_query_param('InstanceId', InstanceId)

    def get_Region(self):
        return self.get_query_params().get('Region')

    def set_Region(self, Region):
        self.add_query_param('Region', Region)


class DescribeDomainConfigRequest(RpcRequest):
    """
    WAF API: DescribeDomainConfig
    Write due to lack of Aliyun WAF SDK
    """
    def __init__(self):
        RpcRequest.__init__(self, 'Waf', '2018-01-17', \
            'DescribeDomainConfig', 'waf', protocol='https')

    def get_InstanceId(self):
        return self.get_query_params().get('InstanceId')

    def set_InstanceId(self, InstanceId):
        self.add_query_param('InstanceId', InstanceId)

    def get_Region(self):
        return self.get_query_params().get('Region')

    def set_Region(self, Region):
        self.add_query_param('Region', Region)

    def get_Domain(self):
        return self.get_query_params().get('Domain')

    def set_Domain(self, Domain):
        self.add_query_param('Domain', Domain)


class AliyunWaf(BaseAliyunApi):
    """
    : Aliyun WAF related API
    """
    def get_domain_names(self, instance_id, region):
        request = DescribeDomainNamesRequest()
        request.set_InstanceId(instance_id)
        request.set_Region(region)
        result = self.client.do_action(request)
        return result

    def get_domain_config(self, instance_id, region, domain_name):
        request = DescribeDomainConfigRequest()
        request.set_InstanceId(instance_id)
        request.set_Region(region)
        request.set_Domain(domain_name)
        result = self.client.do_action(request)
        return result



# pylint: disable=unused-argument
import os
import inspect
import json
import logging
import oss2

from aliyun.log import LogClient
from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.acs_exception.exceptions import ClientException, ServerException
from . import monkey
from .api.base import BaseAliyunApi
from ..exceptions import AliyunClientException

logger = logging.getLogger(__name__)

monkey.patch_all()


def _is_api_endpoint(obj):
    return isinstance(obj, BaseAliyunApi)


class BaseAliyunClient:

    def __new__(cls, *args, **kwargs):
        self = super(BaseAliyunClient, cls).__new__(cls)
        api_endpoints = inspect.getmembers(self, _is_api_endpoint)
        for name, api in api_endpoints:
            api_cls = type(api)
            api = api_cls(self)
            setattr(self, name, api)
        return self

    def __init__(self, access_key=None, access_secret=None, region_id='cn-beijing', **kwargs):
        self._acs_client = AcsClient(access_key, access_secret, region_id=region_id, **kwargs)
        self._log_client = LogClient(
            os.environ.get('ALIYUN_LOG_SAMPLE_ENDPOINT', 'cn-beijing.log.aliyuncs.com'),
            access_key, access_secret
        )
        self._oss_auth = oss2.Auth(access_key, access_secret)

    def _decode_result(self, res):
        try:
            result = json.loads(res, strict=False)
        except (TypeError, ValueError):
            return res
        return result

    def do_action(self, request):
        try:
            res = self._acs_client.do_action_with_exception(request)
        except (ClientException, ServerException) as e:
            raise AliyunClientException(e.error_code, e.message)
        result = self._decode_result(res)

        if isinstance(result, (dict, list)):
            return result
        raise AliyunClientException(-1, "Can not decode response as JSON\n{}".format(result))

    @property
    def acs_client(self):
        return self._acs_client

    @property
    def oss_auth(self):
        return self._oss_auth

    @property
    def log_client(self):
        return self._log_client



from .ecs import AliyunEcs
from .cs import AliyunCs
from .slb import AliyunSlb
from .vpc import AliyunVpc
from .dns import AliyunDns
from .ram import AliyunRam
from .bss import AliyunBss
from .rds import AliyunRds
from .redis import AliyunRedis
from .kms import AliyunKms
from .ons import AliyunOns
from .oss import AliyunOss
from .drds import AliyunDrds
from .pvtz import AliyunDnsPrivateZone
from .waf import AliyunWaf
from .log import AliyunLog



from aliyunsdkkms.request.v20160120 import DecryptRequest, EncryptRequest
from .base import BaseAliyunApi


class AliyunKms(BaseAliyunApi):
    def decrypt_blob(self, cipher_blob):
        request = DecryptRequest.DecryptRequest()
        request.set_CiphertextBlob(cipher_blob)
        result = self.client.do_action(request)
        return result.get('Plaintext')

    def encrypt_blob(self, key_id, plain_blob):
        request = EncryptRequest.EncryptRequest()
        request.set_KeyId(key_id)
        request.set_Plaintext(plain_blob)
        result = self.client.do_action(request)
        return result.get('CiphertextBlob')



import time
import datetime
from aliyunsdkbssopenapi.request.v20171214 import QueryBillOverviewRequest, QueryBillRequest, \
    QueryInstanceBillRequest, QueryInstanceGaapCostRequest, \
    QueryOrdersRequest, GetOrderDetailRequest
from .base import BaseAliyunApi
from ..utils import date_to_isoformat_datetime, isoformat_to_datetime, get_logger

__doc__ = """
账单类型：SubscriptionOrder (预付订单)， PayAsYouGoBill (后付账单)， Refund (退款)， Adjustment (调账)
"""

logger = get_logger(__file__)


class AliyunBss(BaseAliyunApi):
    def query_bill(self, billing_cycle=None, product_code=None, \
                   billing_type=None, delay=3, count_only=False, **kwargs):
        """
        查询用户某个账期内结算账单, 默认返回所有
        https://help.aliyun.com/document_detail/100392.html?spm=a2c4g.11186623.6.591.24bd256406Dfsh

        :param product_code: 产品代码
        :param billing_type: 类型SubscriptionOrder／PayAsYouGoBill／Refund／Adjustment
        :param billing_cycle: 账期，YYYY-MM。样例：2018-07
        :param delay: 请求延时间隔, 防止被api限流
        """
        result_list = []
        request = QueryBillRequest.QueryBillRequest()
        request.set_PageSize(100)
        request.set_BillingCycle(billing_cycle)
        if product_code:
            request.set_ProductCode(product_code)
        if billing_type:
            request.set_Type(billing_type)
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        data = result.get('Data', {})
        result_list.extend(data.get("Items", {}).get("Item", []))
        total_count = float(data.get('TotalCount', 0))
        if count_only:
            return total_count
        page_count = float(data.get('PageSize', 1))
        _p = total_count / page_count
        if _p > 1:
            for _p_num in range(2, int(_p) + 2):
                # 超过10页就延长请求间隔, 防止触发限流
                if _p > 10:
                    time.sleep(delay)
                request.set_PageNum(_p_num)
                logger.info('total page %s, now query %s.', (int(_p) + 1), _p_num)
                result = self.client.do_action(request)
                data = result.get('Data', {})
                result_list.extend(data.get('Items', {}).get('Item', []))
        return result_list

    def query_bill_overview(self, billing_cycle=None, **kwargs):
        """
        查询用户某个账期内账单总览信息。
        https://help.aliyun.com/document_detail/100389.html?spm=a2c4g.11186623.6.590.370e534foaqg3K

        :param billing_cycle: 账期，YYYY-MM。样例：2018-07
        """
        result_list = []
        request = QueryBillOverviewRequest.QueryBillOverviewRequest()
        request.set_BillingCycle(billing_cycle)
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        result_list.extend(result.get('Data', {}).get('Items', {}).get('Item', []))
        return result_list

    def query_instance_bill(self, billing_cycle=None, **kwargs):
        """
        查询用户某个账期内所有商品实例的消费汇总信息。
        https://help.aliyun.com/document_detail/100392.html?spm=a2c4g.11186623.6.591.24bd256406Dfsh

        :param billing_cycle: 账期，YYYY-MM。样例：2018-07
        """
        result_list = []
        request = QueryInstanceBillRequest.QueryInstanceBillRequest()
        request.set_PageSize(100)
        request.set_BillingCycle(billing_cycle)
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        data = result.get('Data', {})
        result_list.extend(data.get("Items", {}).get("Item", []))
        total_count = float(data.get('TotalCount', 0))
        page_count = float(data.get('PageSize', 1))
        _p = total_count / page_count
        if _p > 1:
            for _p_num in range(2, int(_p) + 2):
                request.set_PageNum(_p_num)
                result = self.client.do_action(request)
                data = result.get('Data', {})
                result_list.extend(data.get('Items', {}).get('Item', []))
        return result_list

    def query_instance_gaap_cost(self, billing_cycle=None, **kwargs):
        """
        查询实例的月费用分摊
        注：每月初4号中午12:00后可拉取上月全量GAAP账单，此时间前拉取GAAP账单可能不全。
        https://help.aliyun.com/document_detail/87995.html?spm=a2c4g.11186623.6.593.e5de3f80fLqEW2

        :param billing_cycle: 账期，YYYY-MM。样例：2018-07
        """
        request = QueryInstanceGaapCostRequest.QueryInstanceGaapCostRequest()
        request.set_BillingCycle(billing_cycle)
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def query_order(self, **kwargs):
        """
        查询用户或者分销客户订单列表情况, 缺省查询当前时间最近1小时内订单
        https://help.aliyun.com/document_detail/89079.html?spm=a2c4g.11186623.6.603.373f534f8yL3Zp
        """
        result_list = []
        request = QueryOrdersRequest.QueryOrdersRequest()
        request.set_PageSize(100)
        for k, v in kwargs.items():
            if isinstance(v, datetime.date):
                v = date_to_isoformat_datetime(v)
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        data = result.get('Data', {})
        result_list.extend(data.get("OrderList", {}).get("Order", []))
        total_count = float(data.get('TotalCount', 0))
        page_count = float(data.get('PageSize', 1))
        _p = total_count / page_count
        if _p > 1:
            for _p_num in range(2, int(_p) + 2):
                request.set_PageNum(_p_num)
                result = self.client.do_action(request)
                data = result.get('Data', {})
                result_list.extend(data.get('OrderList', {}).get('Order', []))
        for i, item in enumerate(result_list):
            for k, v in item.items():
                if 'Time' in k:
                    result_list[i][k] = isoformat_to_datetime(v).strftime("%Y-%m-%d %H:%M:%S")
        return result_list

    def get_order_detail(self, order_id=None, delay=None):
        """
        查询用户或分销客户某个订单详情信息
        https://help.aliyun.com/document_detail/89084.html?spm=a2c4g.11186623.6.604.67fc6ff0KxXDNn
        :param order_id: 订单号
        :param delay: 请求延时间隔, 防止被api限流
        """
        if delay:
            time.sleep(delay)
        request = GetOrderDetailRequest.GetOrderDetailRequest()
        request.set_OrderId(order_id)
        result = self.client.do_action(request)
        result = result.get('Data', {}).get('OrderList', {}).get('Order', [])
        result = result[0] if result else result
        for k, v in result.items():
            if 'Time' in k:
                result[k] = isoformat_to_datetime(v).strftime("%Y-%m-%d %H:%M:%S")
        return result



from .base import BaseAliyunApi


class AliyunLog(BaseAliyunApi):
    """
    阿里云日志服务
    https://github.com/aliyun/aliyun-log-python-sdk/blob/master/tests/sample.py
    """
    def list_project(self):
        return self.client.log_client.list_project(size=-1).get_projects()

    def list_logstore(self, project_name=None, logstore_name_pattern=None):
        return self.client.log_client.list_logstore(project_name=project_name,
                                                    logstore_name_pattern=logstore_name_pattern, size=-1).get_logstores()

    def delete_logstore(self, project_name, logstore):
        return self.client.log_client.delete_logstore(project_name, logstore)



from .base import BaseAliyunClient
from . import api


class AliyunClient(BaseAliyunClient):
    ecs = api.AliyunEcs()
    cs = api.AliyunCs()
    slb = api.AliyunSlb()
    vpc = api.AliyunVpc()
    dns = api.AliyunDns()
    ram = api.AliyunRam()
    bss = api.AliyunBss()
    rds = api.AliyunRds()
    redis = api.AliyunRedis()
    kms = api.AliyunKms()
    ons = api.AliyunOns()
    oss = api.AliyunOss()
    drds = api.AliyunDrds()
    pvtz = api.AliyunDnsPrivateZone()
    waf = api.AliyunWaf()
    log = api.AliyunLog()



import oss2
from .base import BaseAliyunApi


class AliyunOss(BaseAliyunApi):
    default_endpoint = 'http://oss-cn-beijing.aliyuncs.com'

    def list_bucket(self):
        """
        列出当前所有oss bucket
        https://help.aliyun.com/document_detail/31957.html?spm=a2c4g.11186623.6.1105.4c9e556cYuWTBJ
        :return:
        """
        result_list = []
        service = oss2.Service(self.client.oss_auth, self.default_endpoint)
        result = service.list_buckets()
        buckets = result.buckets
        result_list.extend(buckets)
        if result.is_truncated:
            r = service.list_buckets(marker=result.next_marker)
            result_list.extend(r.buckets)
            while True:
                if r.is_truncated:
                    r = service.list_buckets(marker=result.next_marker)
                    result_list.extend(r.buckets)
                break
        result_list = [r.__dict__ for r in result_list]
        return result_list



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = [l for l in f.read().splitlines() if l]

setup(name='aliyunpy',
      version='1.0.0',
      author='jokimina',
      author_email='jokimina@163.com',
      description='aliyun sdk tools',
      install_requires=requirements,
      packages=find_packages()
)



import re
import json
import yaml
from kubernetes import client as kclient, config

from aliyunsdkcore.request import RoaRequest
from aliyunsdkcs.request.v20151215 import (
    CreateClusterRequest, DescribeClustersRequest,
    DescribeClusterServicesRequest
)
from .base import BaseAliyunApi
from ...exceptions import AliyunClientException
from ..utils import get_logger, MyJsonEncoder

logger = get_logger(__file__)


class CreateK8sClusterRequest(RoaRequest):
    def __init__(self):
        RoaRequest.__init__(self, 'CS', '2015-12-15', 'CreateK8sCluster')
        self.set_uri_pattern('/clusters')
        self.set_method('POST')


class CreateApplicationRequest(RoaRequest):
    def __init__(self):
        RoaRequest.__init__(self, 'CS', '2015-12-15', 'CreateApplication')
        self.set_uri_pattern('/projects')
        self.set_method('POST')


class GetClusterCerts(RoaRequest):
    def __init__(self, cs_id=None):
        if not cs_id:
            raise AliyunClientException('Need cluster id!')
        RoaRequest.__init__(self, 'CS', '2015-12-15', 'CreateApplication')
        self.set_uri_pattern('/k8s/{}/user_config'.format(cs_id))
        self.set_method('GET')


class AliyunCs(BaseAliyunApi):

    def __init__(self, client=None):
        super().__init__(client=client)
        # k8s client
        self._kclients = {}

    def get_client(self, cs_id=None):
        """
        从阿里云api获取证书, 生成对应api client. for kubernetes
        :param cs_id:
        :return:
        """
        _client = self._kclients.get(cs_id, None)
        if not _client:
            request = GetClusterCerts(cs_id=cs_id)
            c = self.client.do_action(request)
            self._kclients[cs_id] = c
            _config = kclient.Configuration()
            _config.verify_ssl = False
            config.kube_config.KubeConfigLoader(
                config_dict=yaml.load(self._kclients.get(cs_id)['config'], Loader=yaml.FullLoader)) \
                .load_and_set(_config)
            _client = kclient.ApiClient(configuration=_config)
            self._kclients[cs_id] = _client
        return _client

    def list_cluster(self, cs_type='all'):
        """
        获取集群信息
        :param cs_type: 集群类型
                        all: 所有类型   aliyun: swarm模式
                        Kubernetes: 原生k8s  ManagedKubernetes: 托管k8s
        :return: 集群列表
        """
        request = DescribeClustersRequest.DescribeClustersRequest()
        result_list = self.client.do_action(request)
        if cs_type != 'all':
            result_list = [r for r in result_list if r.get('cluster_type', '') == cs_type]
        return result_list

    def get_cluster_name_mapper(self):
        return {_c.get('cluster_id', ''): _c.get('name', '') for _c in self.list_clusters()}

    def list_service(self, cs_id='', cs_type='aliyun', without_acs=True):
        """
        列出所有service或pod信息
        :param cs_id: 集群id
        :param cs_type: 集群类型 aliyun/Kubernetes/ManagedKubernetes
        :param without_acs: 不包含阿里云的系统service, 仅限swarm模式有效
        :return:
        """
        result_list = []
        if cs_type == 'aliyun':
            request = DescribeClusterServicesRequest.DescribeClusterServicesRequest()
            request.set_ClusterId(cs_id)
            result_list = self.client.do_action(request)
            if without_acs:
                if result_list:
                    # docker swarm v3无法正确获取到project的值
                    result_list = [_ for _ in result_list if
                                   not re.match(r'^(acs)', _.get('project', 'acs__v3__'))]
        elif 'Kubernetes' in cs_type:
            _client = self.get_client(cs_id=cs_id)
            ext_api = kclient.ExtensionsV1beta1Api(api_client=_client)
            result = ext_api.list_deployment_for_all_namespaces()
            result_list.extend(json.loads(json.dumps(result.items, cls=MyJsonEncoder)))
        return result_list

    def list_ingress(self, cs_id=''):
        """
        列出所有ingress, k8s only
        :param cs_id: 集群id
        :return:
        """
        _client = self.get_client(cs_id=cs_id)
        ext_api = kclient.ExtensionsV1beta1Api(api_client=_client)
        result = ext_api.list_ingress_for_all_namespaces()
        return json.loads(json.dumps(result.items, cls=MyJsonEncoder))

    def get_projects_by_id(self, cs_id):
        pass

    def create_swarm_cluster(self, **kwargs):
        """
        :params:
          https://help.aliyun.com/document_detail/26054.html?spm=a2c4g.11174283.6.759.4abb7a22fS03SN
        :return: cluster, request, task ID
        """
        request = CreateClusterRequest.CreateClusterRequest()
        for k, v in kwargs.items():
            request.add_body_params(k, v)
        result = self.client.do_action(request)
        return result

    def create_k8s_cluster(self, **kwargs):
        """
        :params:
          https://help.aliyun.com/document_detail/87525.html?spm=a2c4g.11186623.6.795.df9139a5lYR57G
        :return: cluster, request, task ID
        """
        request = CreateK8sClusterRequest.CreateK8sClusterRequest()
        for k, v in kwargs.items():
            request.add_body_params(k, v)
        result = self.client.do_action(request)
        return result

    def create_cs_application(self, **kwargs):
        """
        :return: create status
        """
        request = CreateApplicationRequest()
        for k, v in kwargs.items():
            request.add_body_params(k, v)
        result = self.client.do_action(request)
        return result



#-*- coding:utf-8 -*-
from aliyunsdkram.request.v20150501 import ListUsersRequest, \
        CreateUserRequest, CreateLoginProfileRequest, \
        ListAccessKeysRequest
from .base import BaseAliyunApi


class AliyunRam(BaseAliyunApi):
    """
    阿里云RAM访问控制相关API
    """
    def get_users(self):
        """
        :return: 阿里云用户列表
        """
        user_list = []
        request = ListUsersRequest.ListUsersRequest()
        request.set_accept_format('json')
        result = self.client.do_action(request)
        if not result['IsTruncated']:
            return result['Users']['User']
        #
        user_list.extend(result['Users']['User'])
        marker = result['Marker'] if 'Marker' in result.keys()\
                else 'MARKER'
        while result['IsTruncated'] or result['IsTruncated'] == 'true':
            request.set_Marker(marker)
            result = self.client.do_action(request)
            user_list.extend(result['Users']['User'])
            marker = request.get_Marker()
        return user_list

    def get_user_ids(self):
        """
        :return: 用户名及ID字典
        """
        return {user['UserName']:user['UserId'] for user in self.get_users()}

    def get_accesskey_owner(self, key):
        """
        :return: 根据access key查找用户名
        """
        users = self.get_user_ids()
        for username in users.keys():
            request = ListAccessKeysRequest.ListAccessKeysRequest()
            request.set_UserName(username)
            result = self.client.do_action(request)['AccessKeys']['AccessKey']
            for item in result:
                if key == item['AccessKeyId']:
                    return {'UserName': username, 'Status': item['Status']}

    def create_user(self, **kwargs):
        """
        :params: 用户名 显示名 邮箱 手机号码 备注
        """
        request = CreateUserRequest.CreateUserRequest()
        request.set_accept_format('json')
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result

    def create_login_profile(self, **kwargs):
        """
        :params:
            UserName
            Password
            PasswordResetRequired
            MFABindRequired
        """
        request = CreateLoginProfileRequest.CreateLoginProfileRequest()
        request.set_accept_format('json')
        for k, v in kwargs.items():
            request.add_query_param(k, v)
        result = self.client.do_action(request)
        return result
